#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-07-14
#
# extended inventory of Huawei hardware components
#
# 2023-06-02: rewritten for cmk 2.1

from typing import Dict, List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    startswith,
    TableRow,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
    StringTable,
)


def _get_snmp_date(manufactured_date):
    year = (ord(manufactured_date[0]) << 8) + ord(manufactured_date[1])
    month = ord(manufactured_date[2])
    day = ord(manufactured_date[3])
    return f'{year}-{month:02d}-{day:02d}'


def parse_inv_huawei_hw_components(string_table: List[StringTable]) -> Dict:
    phys_entities, huawei_entities = string_table
    section = {}
    phys_dict = {}

    for entry in phys_entities:
        oid_end, description, name, hardware_rev, firmware_rev, software_rev, serial_num, mfg_name = entry
        phys_dict[oid_end] = {}
        phys_dict[oid_end]['description'] = description
        phys_dict[oid_end]['name'] = name
        phys_dict[oid_end]['hardware_revision'] = hardware_rev
        phys_dict[oid_end]['firmware_revision'] = firmware_rev
        phys_dict[oid_end]['software_revision'] = software_rev
        phys_dict[oid_end]['vendor'] = mfg_name
        phys_dict[oid_end]['serial_number'] = serial_num

    for entry in huawei_entities:
        oid_end, bom_id, bom_en_desc, manufactured_date, board_type, optical_vendor_sn = entry
        if bom_en_desc != '':
            section[oid_end] = {}
            section[oid_end]['bom_id'] = bom_id
            section[oid_end]['bom_description'] = bom_en_desc
            section[oid_end]['manufactured_date'] = _get_snmp_date(manufactured_date)
            section[oid_end]['board_type'] = board_type
            section[oid_end].update(phys_dict[oid_end])
            section[oid_end]['serial_number'] = optical_vendor_sn

    return section


def inventory_huawei_hw_components(section: Dict) -> InventoryResult:
    path = ['hardware', 'components', 'huawei']

    for key, value in section.items():
        yield TableRow(
            path=path,
            key_columns={
                'index': key
            },
            inventory_columns=value
        )


register.snmp_section(
    name='inv_huawei_hw_components',
    parse_function=parse_inv_huawei_hw_components,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.2.1.47.1.1.1.1',
            oids=[
                OIDEnd(),
                '2',  # entPhysicalDescr
                '7',  # entPhysicalName
                '8',  # entPhysicalHardwareRev
                '9',  # entPhysicalFirmwareRev
                '10',  # entPhysicalSoftwareRev
                '11',  # entPhysicalSerialNum
                '12',  # entPhysicalMfgName
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2011.5.25.31.1.1',
            oids=[
                OIDEnd(),
                '2.1.1',  # hwEntityBomId
                '2.1.2',  # hwEntityBomEnDesc
                '2.1.4',  # hwEntityManufacturedDate
                '2.1.11',  # hwEntityBoardType
                '3.1.4'  # hwEntityOpticalVendorSn
            ]
        )
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2011')
)

register.inventory_plugin(
    name='inv_huawei_hw_components',
    inventory_function=inventory_huawei_hw_components,
)
